﻿<#
.SYNOPSIS
  The script replaces old certificate used by the Terminal Server component of YSoft SafeQ 5 and YSoft SafeQ 6.
     
.DESCRIPTION
  Script attempts to replace old certificates in older versions of YSoft SafeQ with certificates "SafeQ DS Web Server.pfx" and "SafeQTerminalServer.pfx" from the script folder.

  The script detects location of the YSoft SafeQ installation (specifically Terminal Server folder) based on path to Windows service "YSoftSafeQTerminalServer" or "YSoftSQ-TS".
  The script attempts to copy the new certificates to all known locations (if they exist) while saving old files with suffix .OLD.
  The script replaces the certificates loaded in IIS for the ports 5012 and 5022.

  PowerShell 3.0 or higher is required, current version can be listed by command: $PSVersionTable.PSVersion.Major
   
.NOTES
  Version:        1.2
  Creation Date:  02/22/2021
     
.EXAMPLE
  Run Windows PowerShell as an administrator and launch the command as follows:
  C:\Users\Administrator\Downloads> .\replace_certificates_automatically.ps1
#>
     
#-----------------------------------------------------------[Execution]------------------------------------------------------------
    
Write-Output "Running script version 1.2"

#Admin rights check
If (-NOT ([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole(([System.Security.Principal.SecurityIdentifier]'S-1-5-32-544'))) {
    Write-Warning 'Administrative rights are missing. Please re-run the script as an Administrator.'
    'Press any key to exit the script.' | Out-Host
    Read-Host
    exit
}

#PS version check
if (!($PSVersionTable.PSVersion.Major -ge 3)) {
     Write-Warning 'PowerShell version is lower than required. Update PowerShell at least to version 3 before running the script again.'
    'Press any key to exit the script.' | Out-Host
    Read-Host
    exit   
}
  
# Create function to detect the TS folder location
function detect_TS() {
    # Define the services
    $ServiceList = Get-ChildItem -Path HKLM:\SYSTEM\CurrentControlSet\Services | ? {(($_.Name.Split('\') | Select-Object -Last 1) -like 'YSoft*') -or (($_.Name.Split('\') | Select-Object -Last 1) -like 'YSQ*')} | % {
        Get-ItemProperty $_.PsPath | ? {
             $_.PSChildName -eq "YSoftSQ-TS" `
        -or $_.PSChildName -eq "YSoftSafeQTerminalServer" `
        }
    }
    
    # Find the root directory for TS service
    ForEach ($Service in $ServiceList) {
        $tmp = $Service.ImagePath.Split()[0].Trim('`"')
        $tmp = $tmp.Substring(0,$tmp.LastIndexOf('\')) -Replace ('\\TSClassic\\?','') -Replace ('\\TSNext\\?','')
        $Service | Add-Member -MemberType NoteProperty -Name Path -Value $tmp
        Write-Host "Detected service $($Service.PSChildName) in $($Service.Path)"
    }

    return $ServiceList
}

# Create function to replace existing certificates
function replace_certificates($TSfolder) {
    $subFolders = '\','certificates\','WsdScan.Services\bin\Certificates\','TSClassic\','TSNext\','TSClassic\certificates\','TSClassic\WsdScan.Services\bin\Certificates\','TSNext\certificates\','TSNext\WsdScan.Services\bin\Certificates\'
    $certNames = 'SafeQ DS Web Server.pfx','SafeQTerminalServer.pfx'

    ForEach ($sub1 in $subFolders) {
        If (!(Test-Path $TSfolder\$sub1\)) {continue}

        ForEach ($cert in $certNames) {
            If (!(Test-Path $TSfolder\$sub1\$cert)) {continue}

            Write-host ""
            Write-Host "Creating backup of $TSfolder\$sub1\$cert by adding suffix .OLD"
            Move-Item -Path "$TSfolder\$sub1\$cert" -Destination "$TSfolder\$sub1\$cert.OLD" -Force
            Write-Host "Copying new certificate to $TSfolder\$sub1\$cert"
            Copy-Item "$PSScriptRoot\$cert" -Destination "$TSfolder\$sub1\"
            
            If ("$TSfolder\$sub1" -match 'Certificates' -and "$TSfolder\$sub1" -notmatch 'WsdScan') {
                Write-Host "Copying crt file to $TSfolder\$sub1\SafeQTerminalServer.crt"
                Copy-Item "$PSScriptRoot\SafeQTerminalServer.crt" -Destination "$TSfolder\$sub1\"
            }
        }
    }

    Write-Host "Finished backup and replacement of old certificates."

}

# Creates function to replace certificate loaded by IIS web server
function replace_IIS_certificate() {
    if ( ($(Get-ChildItem -path cert:\LocalMachine\My).Thumbprint | ? {$_ -eq '30CA272A21EB8109247435347CE1952C3000A601'}).Count -eq 0 ){
        Write-Host "Importing a new certificate to MS Windows store."
        &"certutil" -p '""' -importpfx "my" "$PSScriptRoot\SafeQ DS Web Server.pfx"
    } else {
        Write-Host "New certificate is already present in MS Windows, there is no need to add it again."
    }

    Write-Host "Deleting the old certificate from MS Windows store."
    Get-ChildItem -Path Cert:LocalMachine\My | ? {$_.Thumbprint -eq '01b04add7903a57a186e279dc2a750b87669e17e'} | Remove-Item -Force
    Get-ChildItem -Path Cert:\LocalMachine\Root | ? {$_.Thumbprint -eq '01b04add7903a57a186e279dc2a750b87669e17e'} | Remove-Item -Force

    $oldIIScerts = netsh http show sslcert | ? {$_ -match ':(5012|5022|5024)'}
    if ($oldIIScerts.Count -eq 0) {
        "WARN: No Terminal Server bindings in IIS found. This warning can be ignored in case that all Embedded Terminals are deployed with the type NATIVE."
    } else {
        Write-Host "Count of Terminal Server bindings in IIS before the fix: $($oldIIScerts.Count)"
        if ($oldIIScerts -match '5012') {
            Write-Host "Replacing certificate for port 5012 in IIS."
            &"netsh" http delete sslcert ipport=0.0.0.0:5012
            &"netsh" http add sslcert ipport=0.0.0.0:5012 certhash=30ca272a21eb8109247435347ce1952c3000a601 appid=`{aa99a241-fe4b-4fc2-8839-e0f033088636`} clientcertnegotiation=enable
        }
        if ($oldIIScerts -match '5022') {
            Write-Host "Replacing certificate for port 5022 in IIS."
            &"netsh" http delete sslcert ipport=0.0.0.0:5022
            &"netsh" http add sslcert ipport=0.0.0.0:5022 certhash=30ca272a21eb8109247435347ce1952c3000a601 appid=`{aa99a241-fe4b-4fc2-8839-e0f033088636`} clientcertnegotiation=enable
        }
        if ($oldIIScerts -match '5024') { 
            Write-Host "Replacing certificate for port 5024 in IIS."
            &"netsh" http delete sslcert ipport=0.0.0.0:5024 
            &"netsh" http add sslcert ipport=0.0.0.0:5024 certhash=30ca272a21eb8109247435347ce1952c3000a601 appid=`{aa99a241-fe4b-4fc2-8839-e0f033088636`} clientcertnegotiation=enable
        }
        $newIIScerts = netsh http show sslcert | ? {$_ -match '30ca272a21eb8109247435347ce1952c3000a601'}
        Write-Host "Count of Terminal Server bindings in IIS after the fix: $($newIIScerts.Count)"

        if ($newIIScerts.Count -ne $oldIIScerts.Count) {
            Write-Host ""
            Write-Host "ERROR: Count of IIS bindings after the fix is lower than before the fix. Try following manual steps in:" -ForegroundColor Red
            Write-Host "https://portal.ysoft.com/documentation-and-knowledge-base/knowledge-base?page=Embedded_terminals_stopped_working_after_March_11%2C_2021.html" -ForegroundColor Red
            Write-Host "Press any key to terminate the script"
            Read-Host
            throw
        } else {
            Write-Host "Replacement of certificates in IIS finished with success."
        }

    }
}

# Main script logic
Write-Output ""
Write-Output "Starting with the old certificate replacement"
Write-Output "STEP 1 of 4 Attempting to detect location of TS folder automatically."
Write-Output "*********************************************************************"
$ListOfTS = detect_TS


If ($ListOfTS.PSChildName.count -gt 0) {
    Write-Output ""
    Write-Output "STEP 2 of 4 Found some Terminal Server, running certificate replacement."
    Write-Output "************************************************************************"
    ForEach ($TS in $ListOfTS) {
        Write-Output "Running replacement in $($TS.Path)"
        replace_certificates($TS.Path)
    }
} else { 
    Write-Output "No Terminal Server Services found. Check if TS services exist or run replacement manually based on article:"
    Write-Output "https://portal.ysoft.com/documentation-and-knowledge-base/knowledge-base?page=Embedded_terminals_stopped_working_after_March_11%2C_2021.html"
    exit 1
    }

Write-Output ""
Write-Output "STEP 3 of 4 Checking IIS certificate."
Write-Output "*************************************"
replace_IIS_certificate

Write-Output ""
Write-Output "STEP 4 of 4 Restarting Terminal Server services that are marked to start automatically."
Write-Output "***************************************************************************************"
ForEach ($TS in $ListOfTS) {
    If ($TS.Start -eq 2) {
        Write-Output "Restarting $($TS.PSChildName)"
        Restart-Service -Name $TS.PSChildName
    }
}

Write-Output ""
Write-Output "FINISHED. Press any key to end the script."
Read-Host